#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
/* MQSeries product include file                                  */
#include <cmqc.h>
/* Utils include files                                            */
#include "../../../src/file_util.h"   /* file functions           */
#include "../../../src/mq_util.h"     /* MQSeries functions       */
#include "../../../src/str_util.h"    /* string functions         */
/* MQSeries Services include files                                */
#include "../src/keyword.h"
/* Program specific include files                                 */
#include "profile.h"
#include "expedite.h"


PIESENDINFO GetSendInfo(PIESENDINFO SendInfo, char *buffer, long bufferlength)
{
   int i = 0;

   /* Get all supported keywords                                  */
   GetKeyword("FILEID",  SendInfo->FileId,  MAXFILEIDLEN, buffer, bufferlength);
   GetKeyword("SYSID",   SendInfo->SysId,   SYSIDLEN,     buffer, bufferlength);
   GetKeyword("ACCOUNT", SendInfo->Account, ACCOUNTLEN,   buffer, bufferlength);
   GetKeyword("USERID",  SendInfo->UserId,  USERIDLEN,    buffer, bufferlength);
   GetKeyword("CLASS",   SendInfo->Class,   CLASSLEN,     buffer, bufferlength);

   StrTrimI(SendInfo->FileId, MAXFILEIDLEN);

   /* Set MQSeries destination queue information                  */
   memset(SendInfo->DestinationQ, 0, sizeof(SendInfo->DestinationQ));

   memcpy(SendInfo->DestinationQ, StrTrimI(SendInfo->SysId, SYSIDLEN), SYSIDLEN );

   i = strlen(strcat(SendInfo->DestinationQ, "."));
   memcpy(&SendInfo->DestinationQ[i], StrTrimI(SendInfo->Account,ACCOUNTLEN), ACCOUNTLEN );
   
   i = strlen(strcat(SendInfo->DestinationQ, "."));
   memcpy(&SendInfo->DestinationQ[i], StrTrimI(SendInfo->UserId, USERIDLEN), USERIDLEN );

   StrUpper(SendInfo->DestinationQ);

   SendInfo->FileSize = _FileSize(SendInfo->FileId); 
   
   return SendInfo;
}


PIERECVINFO GetRecvInfo(PIERECVINFO RecvInfo, char *buffer, long bufferlength)
{
   int i = 0;

   GetKeyword("FILEID",  RecvInfo->FileId,  MAXFILEIDLEN, buffer, bufferlength);
   GetKeyword("SYSID",   RecvInfo->SysId,   SYSIDLEN,     buffer, bufferlength);
   GetKeyword("ACCOUNT", RecvInfo->Account, ACCOUNTLEN,   buffer, bufferlength);
   GetKeyword("USERID",  RecvInfo->UserId,  USERIDLEN,    buffer, bufferlength);
   GetKeyword("CLASS",   RecvInfo->Class,   CLASSLEN,     buffer, bufferlength);

   StrTrimI(RecvInfo->FileId, MAXFILEIDLEN);

   /*  Compose ReplyToQ from keywords info */
   memset(RecvInfo->ReplyToQ, 0, sizeof(RecvInfo->ReplyToQ));

   memcpy(RecvInfo->ReplyToQ, StrTrimI(RecvInfo->SysId, SYSIDLEN), SYSIDLEN );

   i = strlen(strcat(RecvInfo->ReplyToQ, "."));
   memcpy(&RecvInfo->ReplyToQ[i], StrTrimI(RecvInfo->Account,ACCOUNTLEN), ACCOUNTLEN );
   
   i = strlen(strcat(RecvInfo->ReplyToQ, "."));
   memcpy(&RecvInfo->ReplyToQ[i], StrTrimI(RecvInfo->UserId, USERIDLEN), USERIDLEN );

   StrUpper(RecvInfo->ReplyToQ);
   
   return RecvInfo;
}


int ReceiveFile(char *LogDir, MQHCONN Hconn, MQCHAR48 QName, MQLONG BufferLength, char *FileName, MQBYTE24 MsgId, MQBYTE24 CorrelId)
{
   MQLONG	OpenOptions;							/* MQOPEN options                */
   MQOD     ObjDesc    = { MQOD_DEFAULT };	/* Object Descriptor             */
   MQHOBJ	Hobj;										/* object handle                 */
   MQLONG   CompCode;								/* completion code               */
   MQLONG   Reason;									/* reason code                   */
   MQMD     MsgDesc    = { MQMD_DEFAULT };	/* Message Descriptor            */
	MQGMO    GetMsgOpts = { MQGMO_DEFAULT };	/* get message options           */
	PMQGMO   pgmo       = NULL;
	PMQMD    pmd        = NULL;
   MQLONG   MessageLength;
   PMQBYTE  Buffer     = NULL;

   FILE *pFile = NULL;

   int i = 0;

   /* Initialise storage ....                                             */
	if ( ( Buffer = (MQBYTE *)malloc(sizeof(MQBYTE) * BufferLength) ) == NULL ) {
	   printf("Cannot allocate memory for message buffer!\n");
	   return -1;
	}

	/*   Open the input queue for input exclusive                          */
	OpenOptions = MQOO_INPUT_EXCLUSIVE
	            + MQOO_FAIL_IF_QUIESCING; /* but not if MQM stopping         */

	strncpy(ObjDesc.ObjectName, QName, MQ_Q_NAME_LENGTH);
	
	MQOPEN(Hconn, &ObjDesc, OpenOptions, &Hobj, &CompCode, &Reason);
	WriteLogMQ(LogDir, _MQOPEN, CompCode, Reason);

   if (CompCode == MQCC_FAILED) {
	   printf("unable to open queue for input\n");
		return((int)Reason );
	} /* if (OpenCode == MQCC_FAILED) */


	/* Use a version 2 MQMD incase the message is Segmented/Grouped        */
	MsgDesc.Version = MQMD_VERSION_2 ;

	/* Initialise storage ....                                             */
	pmd  = malloc(sizeof(MQMD));
	pgmo = malloc(sizeof(MQGMO));
  
	/* Set the version number fot the Get Message Options                  */
	GetMsgOpts.Version = MQGMO_VERSION_2;

	/* Set the options for the get calls                                   */
	GetMsgOpts.Options  = MQGMO_NO_WAIT ;
	GetMsgOpts.Options += MQGMO_FAIL_IF_QUIESCING;
   GetMsgOpts.Options += MQGMO_SYNCPOINT;

	/* Set the message desctriptor and get message options to the defaults */
	pmd  = memcpy(pmd,  &MsgDesc,    sizeof(MQMD) );
	pgmo = memcpy(pgmo, &GetMsgOpts, sizeof(MQGMO) );

   memcpy(pmd->MsgId,    MsgId,    sizeof(pmd->MsgId));
   memcpy(pmd->CorrelId, CorrelId, sizeof(pmd->CorrelId));
	
   MQGET(Hconn, Hobj, pmd, pgmo, BufferLength, Buffer, &MessageLength, &CompCode, &Reason);
	
   if (Reason != MQRC_NO_MSG_AVAILABLE)
		WriteLogMQ(LogDir, _MQGET, CompCode, Reason);
	else
		CompCode = MQCC_FAILED;

	if (CompCode != MQCC_FAILED) {

	   if ( ( pFile = fopen(FileName, "w") ) == NULL ) {
		   MQBACK(Hconn, &CompCode, &Reason);
	      WriteLogMQ(LogDir, _MQBACK, CompCode, Reason);
		}
      else {
         for (i = 0; i < MessageLength; i++) {
            if (fputc(Buffer[i], pFile) == EOF) {
			      printf("Error during write to file\n");
               MQBACK(Hconn, &CompCode, &Reason);
	            WriteLogMQ(LogDir, _MQBACK, CompCode, Reason);
               MQCLOSE(Hconn, &Hobj, 0, &CompCode, &Reason);
	            WriteLogMQ(LogDir, _MQCLOSE, CompCode, Reason);
               return -1;
            }
         } /* for (i = 0; i < MessageLength; i++) */

         MQCMIT(Hconn, &CompCode, &Reason);
	      WriteLogMQ(LogDir, _MQCMIT, CompCode, Reason);

		   fclose(pFile);

         free(Buffer);
      }

	} /* if (CompCode != MQCC_FAILED) */

	MQCLOSE(Hconn, &Hobj, 0, &CompCode, &Reason);
	WriteLogMQ(LogDir, _MQCLOSE, CompCode, Reason);

   return 0;
}
   


int GetExpediteBaseFiles(PINITFILE InitFile, char *BaseInFileName, FILE **pBaseIn, char *BaseOutFileName, FILE **pBaseOut)
{
   char LogText[1024];

   AddPath(BaseInFileName, InitFile->expedite, BASEIN);
   
   if ( (*pBaseIn = FileOpen(InitFile->logdir, BaseInFileName, "r")) == NULL )
      return(-1);

   sprintf(LogText, "Expedite %s file found in %s", 
      BASEIN,
      ( (*(InitFile->expedite) == '\0') ? "<current dir>" : InitFile->expedite) );
   WriteLogFile(InitFile->logdir, LOG_INFO, LogText);

   AddPath(BaseOutFileName, InitFile->expedite, BASEOUT);
   
   if ( (*pBaseOut = FileOpen(InitFile->logdir, BaseOutFileName, "w")) == NULL )
      return(-1);

   sprintf(LogText, "Expedite %s file found in %s", 
      BASEOUT,
      ( (*(InitFile->expedite) == '\0') ? "<current dir>" : InitFile->expedite) );
   WriteLogFile(InitFile->logdir, LOG_INFO, LogText);

   return 0;
}


int SendFile(char *LogDir, MQHCONN Hconn, MQCHAR48 QName, MQCHAR48 QMgrName, MQCHAR48 ReplyToQ, MQCHAR48 ReplyToQMgr, MQCHAR8 Format, char *FileName, PMQBYTE Header, MQLONG HeaderLength, PMQBYTE Trailer, MQLONG TrailerLength)
{
   FILE *pFile = NULL;

   MQHOBJ   Hobj = 0;                        /* object handle                 */
   MQOD     ObjDesc    = { MQOD_DEFAULT };	/* Object Descriptor             */
   MQMD     MsgDesc    = { MQMD_DEFAULT };	/* Message Descriptor            */
	MQPMO    PutMsgOpts = { MQPMO_DEFAULT };	/* put message options           */
   PMQMD    pmd        = NULL;

   MQLONG   CompCode;                        /* completion code               */
   MQLONG   Reason;                          /* reason code                   */
   
   PMQBYTE Buffer = NULL;

   int i = -1;

   long Size = _FileSize(FileName);
   
   if ( ( pFile  = FileOpen(LogDir, FileName, "r") ) == NULL )
      return errno;

   Buffer = (char *)malloc(sizeof(char) * (Size + (Header == NULL ? 0 : HeaderLength) + (Trailer == NULL ? 0 : TrailerLength)));

   if(Header != NULL)
      memcpy(Buffer, Header, HeaderLength);

   if(Trailer != NULL)
	   memcpy(&Buffer[HeaderLength + Size], Trailer, TrailerLength);	

	for (i = 0; i < Size; i++) {
      Buffer[i + (Header == NULL ? 0 : HeaderLength)] = (unsigned char)fgetc(pFile);
		   if (feof(pFile))
			   break;
	}

   pmd = (PMQMD)malloc(sizeof(MQMD));

	/* These options cause the MsgId and CorrelId to be replaced, so  */
	/* that there is no need to reset them before each MQPUT          */ 
	/******************************************************************/
	PutMsgOpts.Options = MQPMO_FAIL_IF_QUIESCING
							 + MQPMO_NEW_MSG_ID
							 + MQPMO_NEW_CORREL_ID;

   strncpy(ObjDesc.ObjectName,     QName,    MQ_Q_NAME_LENGTH);
	strncpy(ObjDesc.ObjectQMgrName, QMgrName, MQ_Q_MGR_NAME_LENGTH);

	memcpy(pmd, &MsgDesc, sizeof(MQMD) );

	memcpy(pmd->Format,    Format,   (size_t)MQ_FORMAT_LENGTH);
	strncpy(pmd->ReplyToQ, ReplyToQ, (size_t)MQ_Q_NAME_LENGTH);
	if(ReplyToQMgr[0] != '\0')																	/* 22/05/2001 */
		strncpy(pmd->ReplyToQMgr, ReplyToQMgr, (size_t)MQ_Q_MGR_NAME_LENGTH);	/* 22/05/2001 */

   pmd->Persistence = MQPER_PERSISTENT;

	MQPUT1( Hconn, 
           &ObjDesc,
           pmd,
           &PutMsgOpts,
           i + (Header == NULL ? 0 : HeaderLength) + (Trailer == NULL ? 0 : TrailerLength),
           Buffer,
           &CompCode,
           &Reason );
   WriteLogMQ(LogDir, _MQPUT1, CompCode, Reason);

   free(Buffer);
	fclose(pFile);

   return 0;
}
