static char *sccsid = "mqrecv.c,  Version 1 Release 5, 23/05/2001";
#define TITLE "MQSeries Services receive program for Expedite"
/******************************************************************/
/*                                                                */
/* (c) Copyright IBM Corp. 2000  All rights reserved.             */
/*                                                                */
/* This sample program is owned by International Business Machines*/
/* Corporation or one of its subsidiaries ("IBM") and is          */
/* copyrighted and licensed, not sold.                            */
/*                                                                */
/* You may copy, modify, and distribute this sample program in    */
/* any form without payment to IBM,  for any purpose including    */
/* developing, using, marketing or distributing programs that     */
/* include or are derivative works of the sample program.         */
/*                                                                */
/* The sample program is provided to you on an "AS IS" basis,     */
/* without warranty of any kind. IBM HEREBY  EXPRESSLY DISCLAIMS  */
/* ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, INCLUDING, BUT NOT  */
/* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND      */
/* FITNESS FOR A PARTICULAR PURPOSE.                              */
/* Some jurisdictions do not allow for the exclusion or limitation*/
/* of implied warranties, so the above limitations or exclusions  */
/* may not apply to you. IBM shall not be liable for any damages  */
/* you suffer as a result of using, modifying or distributing the */
/* sample program or its derivatives.                             */
/*                                                                */
/* Each copy of any portion of this sample program or any         */
/* derivative work, must include the above copyright notice and   */
/* disclaimer of warranty.                                        */
/*                                                                */
/******************************************************************/
/*                                                                */
/* Program name: mqrecv                                           */
/*                                                                */
/* Description: Receive files using Expedite profiles             */
/*                                                                */
/******************************************************************/
/*                                                                */
/* Version 1.0, 21/07/2000, First release NT                      */
/* Version 1.1, 24/07/2000, Commitment control for receive file   */
/* Version 1.2, 27/07/2000, MQSIE header CLASS keyword support    */
/* Version 1.3, 02/08/2000, First release AIX                     */
/* Version 1.4, 22/08/2000, Cleanup for SupportPac release        */
/* Version 1.5, 23/05/2001, First release OS/400                  */
/*                                                                */
/******************************************************************/
#include <stdio.h>                    /* ANSI                     */
#include <stdlib.h>                   /* ANSI                     */
#include <string.h>                   /* ANSI                     */
/* MQSeries product include file                                  */
#include <cmqc.h>                     /* MQSeries product         */
/* Utils include files                                            */
#include "../../../src/file_util.h"   /* file functions           */
#include "../../../src/mq_util.h"     /* MQSeries functions       */
#include "../../../src/str_util.h"    /* string functions         */
/* MQSeries Services include files                                */
#include "../src/getmailboxindex.h"
#include "../src/keyword.h"
/* Expedite specific include file                                 */
#include "expedite.h"
/* Program specific include files                                 */
#include "profile.h"
#include "mqrecv.h"

int main(int argc, char *argv[])
{
   char ProfileName[PATH_MAX];
   /* Expedite files variables                                    */
   char BaseInFileName[PATH_MAX];
   char BaseOutFileName[PATH_MAX];

   char LogText[1024];

   PINITFILE     InitFile    = NULL;
   PIERECVINFO   RecvInfo    = NULL;
   PMQMSGINFO   *RecvMsgInfo = NULL;
   /* MQSeries variables                                          */
   MQHCONN  Hcon = MQHC_UNUSABLE_HCONN;/* connection handle       */
   MQLONG   Reason;                    /* reason code             */
   MQLONG   Qdepth = -1;

   char LineBuffer[255];

   FILE *pBaseIn  = NULL;
   FILE *pBaseOut = NULL;
   
   int i         = 0;
   int LineCount = 0;

   strncpy(ProfileName, argc == 1 ? "profile" : argv[1], PATH_MAX);

   if (( InitFile = ReadProfile(ProfileName)) == NULL )
      return(-1);
   
   if (LogInitFile(InitFile, TITLE) != 0) {
      free(InitFile);
      return(-1);
   }

   if ( (Reason = GetExpediteBaseFiles(InitFile, BaseInFileName, &pBaseIn, BaseOutFileName, &pBaseOut)) != 0) {
      free(InitFile);
      return Reason;
   }

   if ( (RecvInfo = (PIERECVINFO)malloc(sizeof(IERECVINFO))) == NULL ) {
      fprintf(stderr, "Cannot allocate memory !\n");
      free(InitFile);
      return(-1);
   }

   if ( (Reason = MQconn(InitFile->logdir, InitFile->qmgr, &Hcon) ) != 0 ) {
      free(InitFile);
      free(RecvInfo);
      return Reason;
   }

   if ( (RecvMsgInfo = GetMailBoxIndex(InitFile->logdir, Hcon, InitFile->replyq, &Qdepth) ) == NULL ) {
      if (Qdepth < 0) {
         free(InitFile);
         free(RecvInfo);
         return(-1);
      }
   }
   
   if (Qdepth == 0) {
      sprintf(LogText, "There are no messages in %s", InitFile->replyq);
      WriteLogFile(InitFile->logdir, LOG_INFO, LogText);
   } /* if (Qdepth == 0) */
   else {
      sprintf(LogText, "Queue depth for %s : %d message%s", InitFile->replyq, Qdepth, PLURAL(Qdepth) );
      WriteLogFile(InitFile->logdir, LOG_INFO, LogText);      

      LineCount = 0;

      while ( fgets(LineBuffer, sizeof(LineBuffer) - 1, pBaseIn) ) {
         LineCount++;
         /* copy entry into baseout.msg */
         fputs(LineBuffer,pBaseOut);
         fflush(pBaseOut);
         LineBuffer[strlen(LineBuffer) - 1] = '\0';

         if (LineBuffer[0] == '#') {
            sprintf(LogText,"Ignoring line #%d, # at the beginning", LineCount);
            WriteLogFile(InitFile->logdir, LOG_INFO, LogText);
            continue;
         } /* if (LineBuffer[0] == '#') */

         if(FindKeyWord(LineBuffer, strlen(LineBuffer), "RECEIVE") != NULL) {
            /* Get all the supported keywords                          */
            GetRecvInfo(RecvInfo, LineBuffer, strlen(LineBuffer));

            /* Compare queue index table with basein.msg RECEIVE entry */
            for (i = 0; i < Qdepth ; i++) {

               sprintf(LogText,"Processing line #%d, for msg #%d on queue from %.*s", LineCount, i + 1, MQ_Q_NAME_LENGTH, RecvMsgInfo[i]->ReplyToQ );
               WriteLogFile(InitFile->logdir, LOG_INFO, LogText);

               /* entry found ? */
               if (strncmp(RecvInfo->ReplyToQ, RecvMsgInfo[i]->ReplyToQ, MQ_Q_NAME_LENGTH) == 0) {
                  WriteLogFile(InitFile->logdir, LOG_INFO, "Possible message on queue found for this RECEIVE entry");

                  
                  /* Equals CLASS entry with message  */
                  if(!EQUALM(RecvInfo->Class, RecvMsgInfo[i]->Mqsie.Class, CLASSLEN)) {
                     WriteLogFile(InitFile->logdir, LOG_INFO, "No matching CLASS for this RECEIVE entry and current message");
                     continue;
                  } /* if(!EQUALM(RecvInfo->Class, RecvMsgInfo[i]->Mqsie.Class, CLASSLEN)) */
                  else
                     WriteLogFile(InitFile->logdir, LOG_INFO, "Matching CLASS for this RECEIVE entry and current message");

                  /* get message from queue and put it into the named file */
                  if ((Reason = ReceiveFile(InitFile->logdir, Hcon, InitFile->replyq, RecvMsgInfo[i]->DataLength, RecvInfo->FileId, RecvMsgInfo[i]->MsgId, RecvMsgInfo[i]->CorrelId)) != 0) {
                     printf("Cannot receive file %s\n", RecvInfo->FileId);
                     /* create entry in baseout.msg */
                     sprintf(LineBuffer, "NOT RECEIVED;\nRETURN(0%d);\n", Reason);
                     fputs(LineBuffer,pBaseOut);
                     fflush(pBaseOut);
                  }
                  else {
                     sprintf(LogText, "Received file <%s> from <%s>", RecvInfo->FileId, RecvInfo->ReplyToQ);
			            WriteLogFile(InitFile->logdir, LOG_INFO, LogText);
						   /* create entry in baseout.msg */
                     sprintf(LineBuffer, "RECEIVE LENGTH(%ld);\nRETURN(00000);\n", RecvMsgInfo[i]->DataLength);
                     fputs(LineBuffer,pBaseOut);
                     fflush(pBaseOut);
                  }

                  break;

               } /* if (strncmp(RecvMsgInfo[i]->ReplyToQ, RecvInfo->ReplyToQ, MQ_Q_NAME_LENGTH) == 0) */
               else
                  WriteLogFile(InitFile->logdir, LOG_WARNING, "Wrong recipient");

            } /* for (i = 0; i < Qdepth ; i++) { */

         } /* (FindKeyWord(LineBuffer, "RECEIVE") != NULL) */
         else {
            sprintf(LogText,"Ignoring line #%d, no RECV keyword not found", LineCount);
            WriteLogFile(InitFile->logdir, LOG_INFO, LogText);
         }
      
      } /* while ( fgets(buffer, sizeof(buffer) - 1, pBaseIn) ) */
   }

   MQdisc(InitFile->logdir, &Hcon);

   free(InitFile);
   free(RecvInfo);
   free(RecvMsgInfo);

   return 0;
}
